@@ -2,7 +2,7 @@
     gestion.views, un module pour SLM
     - rendu des pages web à l'aide de modèles
 
-    Copyright (C) 2023 Georges Khaznadar <georgesk@debian.org>
+    Copyright (C) 2023-2024 Georges Khaznadar <georgesk@debian.org>
 
     This program is free software: you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -18,11 +18,14 @@
     along with this program.  If not, see <http://www.gnu.org/licenses/>.
 """
 
-from django.shortcuts import render, redirect
+from django.shortcuts import render, redirect, HttpResponse
 from django.db.models import Q, Count
 from django.views.decorators.csrf import ensure_csrf_cookie
 import django.db.models.fields.related
 from django.utils.safestring import mark_safe
+from django.utils.translation import gettext_lazy as _
+from django.utils.translation import gettext
+from django.utils.translation import activate, get_language_info
 
 from .views_common import render_common, unaccent
 # from .views_download import *
@@ -30,10 +33,11 @@ from .views_common import render_common, unaccent
 # from .odfviews import *
 from .models import Eleves, Materiel, LivreSupplement, Jeton, Inventaire, \
     Courriers, Classes, les_modeles, modeles_ordre, ETAT_MATERIEL_CHOIX, \
-    Parametres, Prets, Niveaux
+    Parametres, Prets, Niveaux, NOCLASS, Disciplines, Filieres, Series, \
+    TypeMateriel, Commandes, Personnels, Details
 from .forms import Eleves_Form, Courriers_Form, donnees_utilisateur_Form, \
     Eleves_ancien_Form, Parametres_Form, Materiel_Form
-from .decorators import assure_jeton_valide
+from .decorators import assure_jeton_valide, i18n
 from .utils import svg_histogram, svg_actions_datees, urlToQrcode
 from manuels.settings import MEDIA_ROOT, MEDIA_URL
 
@@ -62,37 +66,75 @@ DATE111 = date.fromisoformat('0001-01-01')
 # Create your views here.
 
 @ensure_csrf_cookie
+@i18n
 def index(request):
     return redirect("/apropos#apropos")
 
+@i18n
+def jsi18n(request):
+    """
+    renvoie un catalogue Javascript pour certaines traductions courtes,
+    sans contexte
+    """
+    translations = {
+        'Oui': gettext('Oui'),
+        'Non': gettext('Non'),
+        'OK': gettext('OK'),
+        'Valider': gettext('Valider'),
+        'Echappement': gettext('Échappement'),
+        'Supprimer': gettext('Supprimer'),
+        'ERREUR': gettext('ERREUR'),
+        'erreurs': gettext("Des erreurs ont été détectées"),
+        'date_erreur': gettext("Date mal formée"),
+        'Attention': gettext('Attention'),
+        'Bof': gettext('Quelque chose s\'est passé de travers.'),
+        'Confirm': gettext("Veuillez confirmer"),
+        'Attendre': gettext('Attendre ...'),
+        'Neuf': gettext('Neuf'),
+        'Bon': gettext('Bon'),
+        'Autre': gettext('Autre'),
+        "code_inconnu": gettext("code inconnu ?"),
+    }
+    return HttpResponse(
+        "const I18N = " + json.dumps(translations),
+        content_type="application/javascript"
+    )
+
 @ensure_csrf_cookie
+@i18n
 def parametres(request):
     p = Parametres.objects.all().order_by(
         '-AnneeScolaire')
-    return render_common(request, 'gestion/parametres.html', "Paramètres",{
+    return render_common(request, 'gestion/parametres.html',
+                         _("Paramètres"),{
          "aide": "parametres",
          "param": p[0] if p else None,
      })
 
 @ensure_csrf_cookie
+@i18n
 def apropos(request):
       return render_common(
-          request, 'gestion/apropos.html', "À propos", pour_tous = True)
+          request, 'gestion/apropos.html', _("À propos"), pour_tous = True)
     
 @ensure_csrf_cookie
+@i18n
 def assistance(request):
-      return render_common(request, 'gestion/assistance.html', "Assistance", {
+      return render_common(request, 'gestion/assistance.html',
+                           _("Assistance"), {
           "aide": "assistance",
       })
     
 @ensure_csrf_cookie
+@i18n
 def achats_inventaire(request):
       return render_common(request, 'gestion/achats_inventaire.html',
-                           "Achats/inventaire",{
+                           _("Achats/inventaire"),{
                                "aide": "achats_inventaire",
                            })
     
 @ensure_csrf_cookie
+@i18n
 def prets(request):
     """
     Page de gestion des livres prêtés à un élève ;
@@ -100,12 +142,14 @@ def prets(request):
     de l'élève
     """
     nomprenom = request.POST.get("nomprenom","")
-    return render_common(request, 'gestion/prets.html', "Livres prêtés, à ...",{
+    return render_common(request, 'gestion/prets.html',
+                         _("Livres prêtés, à ..."),{
         "nomprenom": nomprenom,
         "aide": "prets",
     })
 
 @ensure_csrf_cookie
+@i18n
 def stats(request):
     """
     Affiche diverses statistiques
@@ -140,8 +184,9 @@ def stats(request):
             return ", ".join(sorted(list(self.titres)))
         
         def __str__(self):
-            return f"""\
-Valeurs globales : {self.volume} m³, {self.masse} kg, {self.les_titres}"""
+            return _("""\
+Valeurs globales : {volume} m³, {masse} kg, {les_titres}""").\
+format(**self.__dict__)
         
     volumes = {}
     total = GlobalLivre()
@@ -160,8 +205,8 @@ Valeurs globales : {self.volume} m³, {self.masse} kg, {self.les_titres}"""
     livres_par_eleves = [e.prets_en_cours for e in membres]
     histogramme_lpe = svg_histogram(
         livres_par_eleves,
-        "Répartition des nombres de livres pour chaque élève",
-        "livres",
+        _("Répartition des nombres de livres pour chaque élève"),
+        _("livres"),
         bins = [0,1,2,3,4,5,6,7,8,9,10,11,12,13,14,15,16],
     )
     graphique_prets = None
@@ -169,11 +214,11 @@ Valeurs globales : {self.volume} m³, {self.masse} kg, {self.les_titres}"""
     pr = Prets.objects.all()
     if pr:
         graphique_prets = svg_actions_datees(
-            (p.date_pret for p in pr), "date de prêt")
+            (p.date_pret for p in pr), _("date de prêt"))
         graphique_retours = svg_actions_datees(
             (p.date_retour for p in pr.filter(
                 ~Q(date_retour = DATE111))),
-            "date de retour"
+            _("date de retour")
         )
     
     return render_common(request,'gestion/stats.html', "Statistiques ...", {
@@ -189,6 +234,7 @@ Valeurs globales : {self.volume} m³, {self.masse} kg, {self.les_titres}"""
     })
 
 @ensure_csrf_cookie
+@i18n
 def inventaire(request):
     """
     Implémente la page d'inventaire
@@ -196,7 +242,8 @@ def inventaire(request):
     niveaux = Niveaux.objects.filter(
         rang__gte = "E", rang__lte = "H").order_by("rang")
     categories = Classes.categories()
-    return render_common(request,'gestion/inventaire.html', "Inventaire",{
+    return render_common(request,'gestion/inventaire.html',
+                         _("Inventaire"),{
         "aide": "achats_inventaire#inventaire",
         "categories": categories,
         "niveaux": niveaux,
@@ -204,6 +251,7 @@ def inventaire(request):
     })
 
 @ensure_csrf_cookie
+@i18n
 def constantes(request):
     constantes = Parametres.objects.all().order_by("AnneeScolaire")
     messages = Courriers.objects.all().order_by("N")
@@ -212,7 +260,7 @@ def constantes(request):
                for c in Classes.classes_importantes()}
     return render_common(
         request, 'gestion/constantes.html',
-        "Constantes de l'établissement",
+        _("Constantes de l'établissement"),
         {
             "aide": "parametres#constantes",
             "constantes": constantes,
@@ -223,6 +271,7 @@ def constantes(request):
         })
 
 @ensure_csrf_cookie
+@i18n
 def constantes_modif(request):
     """
     Permet de voir et d'éditer la description de l'établissement
@@ -252,7 +301,7 @@ def constantes_modif(request):
     # affichage de l'interface web
     return render_common(
         request,'gestion/constantes_modif.html',
-        "Constantes de l'établissement",
+        _("Constantes de l'établissement"),
         {
             "numero": numero,
             "clonage": clonage,
@@ -261,6 +310,7 @@ def constantes_modif(request):
         })
     
 @ensure_csrf_cookie
+@i18n
 def catalogue(request):
     manuels = list(Materiel.objects.all().order_by("-Annee_Depot_Legal", "titre"))
     for m in manuels:
@@ -276,13 +326,14 @@ def catalogue(request):
             replace("<br><br>", "<br>")
     return render_common(
         request, 'gestion/catalogue.html',
-        "Catalogue de la Coopérative",
+        _("Catalogue de la Coopérative"),
         {
             "aide": "achats_inventaire#catalogue",
             "manuels": manuels,
         })
 
 @ensure_csrf_cookie
+@i18n
 def catalogue_modif(request):
     """
     Permet de voir et d'éditer les entrées du catalogue
@@ -323,7 +374,7 @@ def catalogue_modif(request):
     options = sorted([ o for o in options if o])
     return render_common(
         request,'gestion/catalogue_modif.html',
-        "Gestion d'un des items du catalogue",
+        _("Gestion d'un des items du catalogue"),
         {
             "numero": numero,
             "categories": categories,
@@ -334,6 +385,7 @@ def catalogue_modif(request):
         })
     
 @ensure_csrf_cookie
+@i18n
 def purger_prets(request):
     """
     Permet de purger les prêts quand le livre est revenu
@@ -351,7 +403,7 @@ def purger_prets(request):
             date_retour__lt = date2)
     return render_common(
         request,'gestion/purger_prets.html',
-        "Purger les prêts anciens",
+        _("Purger les prêts anciens"),
         {
             "aide": "parametres#purger",
             "prets_rendus" : prets_rendus,
@@ -360,6 +412,7 @@ def purger_prets(request):
         })
 
 @ensure_csrf_cookie
+@i18n
 def siecle(request):
     """
     Import d'un fichier siecle, éventuellement POSTé (param fichier_siecle)
@@ -372,13 +425,14 @@ def siecle(request):
     }
     return render_common(
         request,'gestion/siecle.html',
-        "Importation depuis la base de données SIÈCLE",
+        _("Importation depuis la base de données SIÈCLE"),
         {
             "aide": "parametres#siecle",
             "liste_doublons_SLM": liste_doublons_SLM,
         })
     
 @ensure_csrf_cookie
+@i18n
 def eleves(request):
     """
     Affiche la page de gestion des enregistrements des élèves ;
@@ -463,7 +517,7 @@ def eleves(request):
         # aucun éléve, aucune classe ne sont demandés
         result= render_common(
             request,'gestion/eleves.html',
-            "Gestion de la base Élèves",
+            _("Gestion de la base Élèves"),
             {
                 "aide": "parametres#base_eleves",
                 "saved": saved,
@@ -471,6 +525,7 @@ def eleves(request):
     return result
             
 @ensure_csrf_cookie
+@i18n
 def cartes(request):
     """
     Affiche la page de gestion des cartes de membres ;
@@ -487,7 +542,7 @@ def cartes(request):
     classes = Classes.objects.filter(compte__gt=0).filter(Qexpr)
     result= render_common(
         request,'gestion/cartes.html',
-        "Gestion des cartes de membre",
+        _("Gestion des cartes de membre"),
         {
             "aide": "parametres#cartes_membre",
             "classes": classes,
@@ -497,22 +552,25 @@ def cartes(request):
             
     
 @ensure_csrf_cookie
+@i18n
 def bon_de_commande(request):
     return render_common(
         request,'gestion/bon_de_commande.html',
-        "Bons de commande",
+        _("Bons de commande"),
         {
         })
     
 @ensure_csrf_cookie
+@i18n
 def tarif(request):
     return render_common(
         request,'gestion/tarif.html',
-        "Définition des tarifs",
+        _("Définition des tarifs"),
         {
         })
     
 @ensure_csrf_cookie
+@i18n
 def codes_barres(request):
     """
     Gestion de codes-barres pour un manuel du catalogue.
@@ -531,7 +589,7 @@ def codes_barres(request):
         etats = [e[1] for e in ETAT_MATERIEL_CHOIX]
     return render_common(
         request,'gestion/codes_barres.html',
-        "Inventaire des manuels / Impression des étiquettes à codes-barres",
+        _("Inventaire des manuels / Impression des étiquettes à codes-barres"),
         {
             "aide": "achats_inventaire#ajout_codes",
             "manuel": manuel,
@@ -540,6 +598,7 @@ def codes_barres(request):
         })
     
 @ensure_csrf_cookie
+@i18n
 def codes_barres_print(request):
     """
     Impression de codes-barres pour un manuel du catalogue.
@@ -560,7 +619,7 @@ def codes_barres_print(request):
               if k.startswith("dejavu-")]
     try:
         date_dejavu = date.fromisoformat(date_dejavu[0])
-    except ValueError:
+    except (ValueError, IndexError):
         date_dejavu = ""
     codes = []
     if date_dejavu:
@@ -589,7 +648,7 @@ def codes_barres_print(request):
     manuel = Materiel.objects.get(id = id_catalogue)
     return render_common(
         request,'gestion/codes_barres_print.html',
-        "Impression de codes-barres",
+        _("Impression de codes-barres"),
         {
             "aide": "achats_inventaire#impression_codes",
             "manuel": manuel,
@@ -600,6 +659,7 @@ def codes_barres_print(request):
         })
 
 @ensure_csrf_cookie
+@i18n
 def cartes_membres_print(request):
     """
     Impression de cartes de membres.
@@ -620,7 +680,8 @@ def cartes_membres_print(request):
         id_eleves = [Eleves.objects.get(NomPrenomEleve = nomprenom).id]
     else:
         id_eleves = json.loads(request.POST.get("id_eleves","[]"))
-        classes = json.loads(request.POST.get("classes_chosen","[]"))
+        classes = [ c.strip() for c in
+                    json.loads(request.POST.get("classes_chosen","[]"))]
     if max_nb < 2:
         max_nb=2
     if max_nb > 8:
@@ -679,7 +740,7 @@ def cartes_membres_print(request):
     id_eleves = json.dumps([e.pk for e in eleves])
     return render_common(
         request,'gestion/cartes_membres_print.html',
-        "Impression de cartes de membres",
+        _("Impression de cartes de membres"),
         {
             "aide": "parametres#cartes_membre_selection",
             "classes" : json.dumps(classes),
@@ -692,6 +753,7 @@ def cartes_membres_print(request):
             "derniere": cartes[-1]["page"],
         })
 
+@i18n
 def cartes_membres_print_verso(request):
     """
     Impression de versos de cartes de membres.
@@ -701,7 +763,7 @@ def cartes_membres_print_verso(request):
     nb = int(request.POST.get("nb", "1"))
     return render_common(
         request,'gestion/cartes_membres_print_verso.html',
-        "Impression des versos de cartes de membre",
+        _("Impression des versos de cartes de membre"),
         {
             "aide": "parametres#cartes_verso",
             "nb": nb,
@@ -709,6 +771,7 @@ def cartes_membres_print_verso(request):
         })
 
 @ensure_csrf_cookie
+@i18n
 def retours(request):
     """
     Page de retour des livres
@@ -718,7 +781,7 @@ def retours(request):
     livre = request.POST.get("livre", "")
     return render_common(
         request,'gestion/retours.html',
-        "Gestion des retours de manuels",
+        _("Gestion des retours de manuels"),
         {
             "aide": "retours",
             "nomprenom": nomprenom,
@@ -726,6 +789,7 @@ def retours(request):
         })
 
 @ensure_csrf_cookie
+@i18n
 def sauvegarde(request):
 
     def sauvegarde_date(path):
@@ -743,7 +807,7 @@ def sauvegarde(request):
     }
     return render_common(
         request,'gestion/sauvegarde.html',
-        "Sauvegarder la base de données SLM",
+        _("Sauvegarder la base de données SLM"),
         {
             "aide": "assistance#sauvegarde",
             "sauvegardes": sorted(
@@ -753,6 +817,7 @@ def sauvegarde(request):
         })
     
 @ensure_csrf_cookie
+@i18n
 def messages_modif(request):
     """
     Permet de voir et d'éditer les messages prédéfinis
@@ -781,7 +846,7 @@ def messages_modif(request):
     # affichage de l'interface web
     return render_common(
         request,'gestion/messages_modif.html',
-        "Les messages prédéfinis",
+        _("Les messages prédéfinis"),
         {
             "numero": numero,
             "clonage": clonage,
@@ -789,6 +854,7 @@ def messages_modif(request):
             "saved": saved,
         })
     
+@i18n
 def donnees_utilisateur(request):
     """
     Modification des données utilisateur
@@ -813,11 +879,13 @@ def donnees_utilisateur(request):
         })
     return render_common(
         request,'gestion/donnees_utilisateur.html',
-        "Mise à jour des données personnelles pour " + request.user.username,
+        _("Mise à jour des données personnelles pour {user}").
+        format(user = request.user.username),
         {
             "form": form,
         })
 
+@i18n
 def importeFromGemaSCO(request):
     """
     Importation de fichiers issus de GemaSCO
@@ -825,6 +893,7 @@ def importeFromGemaSCO(request):
     modeles = {}
     for mod in modeles_ordre:
         # création d'un arbre de dépendances
+        print("GRRRRR mod =", mod)
         the_class = eval(mod)
         related = [f.related_model.__name__ for f in the_class._meta.fields \
                    if isinstance(f, django.db.models.fields.related.ForeignKey)]
@@ -835,13 +904,14 @@ def importeFromGemaSCO(request):
         }
     return render_common(
         request,'gestion/importeFromGemaSCO.html',
-        "Importation de données de GemaSCO",
+        _("Importation de données de GemaSCO"),
         {
             "modeles": modeles,
             "names": list(modeles.keys()),
         })
 
 @ensure_csrf_cookie
+@i18n
 def inscription_provisoire(request):
     """
     Affiche la page pour l'inscription provisoire des élèves ;
@@ -860,7 +930,7 @@ def inscription_provisoire(request):
             eleve = Eleves(
                 Nom_de_famille = "À changer",
                 Prenom = "À changer",
-                classe = Classes.objects.get(libelle="PAS_DE_CLASSE"),
+                classe = Classes.objects.get(libelle=NOCLASS),
                 autorisation_provisoire_inscription=code_temporaire,
             )
             eleve.save()
@@ -895,6 +965,7 @@ def inscription_provisoire(request):
             "eleve": eleve,
         })
     
+@i18n
 def invite_inscription_provisoire(request):
     """
     Crée une autorisation temporaire pour saisir les coordonnées d'un nouvel
@@ -930,7 +1001,7 @@ def invite_inscription_provisoire(request):
         '<?xml version="1.0" encoding="UTF-8"?>', '')
     return render_common(
         request, "gestion/invite_inscription_provisoire.html",
-        "Invitation à l'auto-inscription",
+        _("Invitation à l'auto-inscription"),
         {
             "aide": "auto_inscription",
             "url": url,
@@ -940,6 +1011,7 @@ def invite_inscription_provisoire(request):
         })
 
 @ensure_csrf_cookie
+@i18n
 def print_etat_classe(request):
     """
     produit un état des prêts par élèves pour une classe,
@@ -960,6 +1032,7 @@ def print_etat_classe(request):
     )
     
 @ensure_csrf_cookie
+@i18n
 def livres_pour(request):
     """
     Présente une page imprimable, pour les livres correspondant à une
@@ -981,6 +1054,7 @@ def livres_pour(request):
    
     
 @ensure_csrf_cookie
+@i18n
 def cautions(request):
     """
     Sert la page qui permet de gérer les cautions versées par les élèves
@@ -1000,7 +1074,7 @@ def cautions(request):
         else:
             msg = f"L'élève « {nomprenom} » n'a pas été trouvé dans la base"
     return render_common(
-        request, 'gestion/cautions.html', "Cautions",
+        request, 'gestion/cautions.html', _("Cautions"),
         {
             "aide": "cautions",
             "eleve": eleve,
@@ -1010,28 +1084,31 @@ def cautions(request):
         }
     )
     
+@i18n
 def revue_du_stock(request):
     """
     Méthode d'inventaire adaptée à la revue du stock
     """
     return render_common(
-        request, 'gestion/revue_du_stock.html', "Revue du stock",
+        request, 'gestion/revue_du_stock.html', _("Revue du stock"),
         {
             "aide": "revue_du_stock",
         }
     )
     
+@i18n
 def eleve_scan(request):
      """
     Page qui permet aux élèves de faire leur "shopping" dans la réserve
      """
      return render_common(
-        request, 'gestion/eleve_scan.html', "Recherche de livres",
+        request, 'gestion/eleve_scan.html', _("Recherche de livres"),
          {
              "aide": "eleve_scan",
          }
      )
 
+@i18n
 def eleve_invite(request):
     """
     Page qui permet aux élèves de visiter la réserve comme une boutique.
@@ -1051,7 +1128,7 @@ def eleve_invite(request):
     baseUrl = f"{request.scheme}://{request.get_host()}/"
     url = baseUrl + f"boutique?jeton={j.id}"
     return render_common(
-        request, 'gestion/eleve_invite.html', "Invitation",
+        request, 'gestion/eleve_invite.html', _("Invitation"),
         {
             "url": url,
             "qrcode": urlToQrcode(url),
@@ -1060,6 +1137,7 @@ def eleve_invite(request):
         }
     )
     
+@i18n
 def boutique(request):
     """
     Entrée dans la Coop en mode boutique
@@ -1074,7 +1152,7 @@ def boutique(request):
     else:
         jeton = None
     return render_common(
-        request, 'gestion/boutique.html', "Boutique de livres",
+        request, 'gestion/boutique.html', _("Boutique de livres"),
         {
             "jeton": jeton,
             "aide": "boutique",
@@ -1082,13 +1160,14 @@ def boutique(request):
         pour_tous = True,
     )
 
+@i18n
 def jeton_perime(request):
     """
     Page qui apparaît quand on visite une page protégée par le décorateur
     @assure_jeton_valide et que le jeton de session est périmé.
     """
     return render_common(
-        request, 'gestion/jeton_perime.html', "Il est trop tard",
+        request, 'gestion/jeton_perime.html', _("Il est trop tard"),
         {
             "jeton": request.session.get("jeton"),
             "aide": "jeton_perime",
@@ -1098,6 +1177,7 @@ def jeton_perime(request):
     
 @assure_jeton_valide
 @ensure_csrf_cookie
+@i18n
 def moisson(request):
     """
     Moisson de livres, par un élève. Le POST contient une liste de livres
@@ -1114,7 +1194,7 @@ def moisson(request):
         ids = [m.id for m in Materiel.objects.filter(abrege = a)]
         tous_manuels.append({"abrege": a, "ids": ids})
     return render_common(
-        request, 'gestion/moisson.html', "Chercher les livres",
+        request, 'gestion/moisson.html', _("Chercher les livres"),
         {
             "jeton": Jeton.objects.get(id = request.session.get("jeton")),
             "manuels": manuels,
@@ -1124,6 +1204,7 @@ def moisson(request):
         pour_tous = True,
     )
 
+@i18n
 def correlation_code(request):
     """
     Affiche du code Python pour entreprendre une analyse de corrélation entre
@@ -1179,7 +1260,8 @@ def correlation_code(request):
     for line in d.split("\n"):
         donnees_formatees += re.sub(r"(([01], ){26})", r"\1\n   ", line)+"\n"
     return render_common(
-        request, 'gestion/correlation_code.html', "Code Python de corrélation",
+        request, 'gestion/correlation_code.html',
+        _("Code Python de corrélation"),
         {
             "options": options,
             "abreges": abreges,
@@ -1189,18 +1271,21 @@ def correlation_code(request):
         },
     )  
 
+@i18n
 def correlation(request):
     """
     Met en évidence des corrélations entre les options et les manuels
     empruntés par les élèves
     """
     return render_common(
-        request, 'gestion/correlation.html', "Corrélation entre options et livres",
+        request, 'gestion/correlation.html',
+        _("Corrélation entre options et livres"),
         {
            "aide": "correlation",
         },
     )  
     
+@i18n
 def qr_boutique(request):
     """
     permet d'imprimer les étiquettes pour les QR-codes de boutique
@@ -1235,7 +1320,8 @@ def qr_boutique(request):
             nombre = len(Inventaire.objects.filter(
                 materiel__abrege = livre_exemple.abrege))
     return render_common(
-        request, 'gestion/qr_boutique.html', "QR-Codes pour le passage en boutique",
+        request, 'gestion/qr_boutique.html',
+        _("QR-Codes pour le passage en boutique"),
         {
             "piles": piles,
             "premiere_pile": premiere_pile,
@@ -1245,6 +1331,7 @@ def qr_boutique(request):
         },
     )  
     
+@i18n
 def prof_livre_invite(request):
     """
     Page qui permet aux profs de proposer, pendant 96 heures, des
@@ -1266,7 +1353,7 @@ def prof_livre_invite(request):
     url = baseUrl + f"livre_supplement?jeton={j.id}"
     livres_supplement = LivreSupplement.objects.all()
     return render_common(
-        request, 'gestion/prof_livre_invite.html', "Livres à emprunter",
+        request, 'gestion/prof_livre_invite.html', _("Livres à emprunter"),
         {
             "url": url,
             "qrcode": urlToQrcode(url),
@@ -1277,6 +1364,7 @@ def prof_livre_invite(request):
     )
     
 @assure_jeton_valide
+@i18n
 def livre_supplement(request):
     return render_common(
         request,
@@ -1288,6 +1376,7 @@ def livre_supplement(request):
         pour_tous = True,
     )
 
+@i18n
 def livre_supplement_go(request):
     data = request.POST.get("data")
     commentaire = request.POST.get("commentaire")
@@ -1300,7 +1389,7 @@ def livre_supplement_go(request):
     return render_common(
         request,
         'gestion/livre_supplement_go.html',
-        "Livre en supplément",
+        _("Livre en supplément"),
         {
             "aide": "livre_supplement_go",
             "data": data,
@@ -1311,6 +1400,7 @@ def livre_supplement_go(request):
         pour_tous = True,
     )
     
+@i18n
 def verif_plan(request, filename):
     """
     Vérifie qu'un plan contient des données correctement organisées. Le plan
@@ -1357,7 +1447,7 @@ def verif_plan(request, filename):
     return render_common(
         request,
         'gestion/verif_plan.html',
-        "Vérification de plan",
+        _("Vérification de plan"),
         {
             "aide": "verif_plan",
             "filename": filename + ".svg",
@@ -1366,6 +1456,7 @@ def verif_plan(request, filename):
             "svgfiles": svgfiles,
         })
 
+@i18n
 def une_etiquette(request, numero):
     """
     Permet d'imprimer une étiquette pour un livre, après choix de la
@@ -1377,7 +1468,7 @@ def une_etiquette(request, numero):
     return render_common(
         request,
         'gestion/une_etiquette.html',
-        "Imprimer une étiquette",
+        _("Imprimer une étiquette"),
         {
             "aide": "une_etiquette",
             "numero" : numero,
@@ -1386,10 +1477,12 @@ def une_etiquette(request, numero):
             "etiquettes_ligs": range(8),
         })
     
-def eleves_plus_3(request, niveau):
+@i18n
+def eleves_plus_n(request, niveau, nombre):
     """
-    Donne une liste des élèves d'un niveau qui doivent plus de trois livres
+    Donne une liste des élèves d'un niveau qui doivent plus de n livres
     @param niveau seconde, premiere ou terminale
+    @param nombre nombre de livres dus au moins (zéro pour au moins un livre)
     """
     debut = "2"
     n=""
@@ -1402,22 +1495,33 @@ def eleves_plus_3(request, niveau):
     elif niveau == "terminales":
         debut = "T"
         n = "terminale"
+    """
     eleves = Eleves.objects.filter(classe__libelle__startswith = debut).\
         order_by("classe__libelle", "sans_accent")
-    plus_3 = {
-        e: len(Prets.faits_a(e)) for e in eleves if len(Prets.faits_a(e)) > 3
+    plus_n = {
+        e: len(Prets.faits_a(e)) for e in eleves if \
+           len(Prets.faits_a(e)) > int(nombre)
+    }
+    """
+    plus_n = {
+        e: len(Prets.faits_a(e)) for e in \
+        Eleves.avec_prets_qs(int(nombre)).\
+        filter(classe__libelle__startswith = debut).\
+        order_by("classe__libelle", "sans_accent")
     }
     return render_common(
         request,
-        'gestion/eleves_plus_3.html',
-        "Élèves qui doivent plus de trois livres",
+        'gestion/eleves_plus_n.html',
+        _("Élèves qui doivent des livres"),
         {
-            "aide": "eleves_plus_3",
-            "eleves": plus_3,
+            "aide": "eleves_plus_n",
+            "eleves": plus_n,
             "n": n,
+            "nombre": nombre,
             "date": date.today(),
         })
         
+@i18n
 def abreges(request, svgfile, multiplicateur):
     """
     Prépare l'impression d'étiquettes pour les abrégés à placarder
@@ -1447,7 +1551,7 @@ def abreges(request, svgfile, multiplicateur):
     return render_common(
         request,
         'gestion/abreges.html',
-        "Imprimer les étiquettes d'abrégés",
+        _("Imprimer les étiquettes d'abrégés"),
         {
             "aide": "abreges",
             "abreges": abreges,
